import numpy as np
from numpy.testing import assert_allclose
from scipy.optimize import curve_fit

from light_curve.light_curve_py import LinearTrend


def test_lintrend_1():
    def f(x, slope, c):
        return x * slope + c

    m = np.array([2.0, 4.0, 7.0, 10.0, 15.0, 28.0])
    t = np.array([1.0, 3.0, 6.0, 10.0, 14.0, 17.0])
    n = len(t)

    feature = LinearTrend()
    actual = feature(t, m, None)

    (slope, c), popt = curve_fit(f, t, m, absolute_sigma=False)
    chisq = np.sum((m - (t * slope + c)) ** 2) / (n - 2)

    desired = (slope, np.sqrt(popt[0][0]), np.sqrt(chisq))
    assert_allclose(actual, desired, rtol=1e-6)


def test_lintrend_2():  # check if sigma is used in least squares
    m = np.linspace(20, 25, 15)
    t = np.linspace(0, 10, 15)
    sigma = np.linspace(0, 1, 15)

    feature = LinearTrend()
    actual = feature(t, m, None)
    desired = feature(t, m, sigma)
    assert_allclose(actual, desired)



import numpy as np

from ._base import BaseFeature


class Amplitude(BaseFeature):
    def _eval(self, t, m, sigma=None):
        return 0.5 * np.ptp(m)

    @property
    def size(self):
        return 1


__all__ = ("Amplitude",)



import numpy as np

from ._base import BaseFeature


class PercentAmplitude(BaseFeature):
    def _eval(self, t, m, sigma=None):
        median = np.median(m)
        return np.max((np.max(m) - median, median - np.min(m)))

    @property
    def size(self):
        return 1


__all__ = ("PercentAmplitude",)



from dataclasses import dataclass
from typing import Tuple

import numpy as np

from ._base import BaseFeature


@dataclass()
class FluxNNotDetBeforeFd(BaseFeature):
    """Number of non-detections before the first detection for measurements of the flux.

    Feature use a user-defined signal to noise ratio to define non-detections and count their number before
    the first detection. strictly_fainter flag allows counting non-detections with a strictly smaller upper limit
    than the first detection flux (there is no such feature in the original article).

    - Depends on: **flux**
    - Minimum number of observations: **2**
    - Number of features: **1**

    Attributes
    ----------
    signal_to_noise : float
        Signal to noise ratio.
    strictly_fainter : bool
        Flag to determine if to find non-detections with strictly smaller upper limit than the first detection flux.

    P. Sánchez-Sáez et al 2021, [DOI:10.3847/1538-3881/abd5c1](https://doi.org/10.3847/1538-3881/abd5c1)
    """

    signal_to_noise: float
    strictly_fainter: bool = False

    def _eval(self, t, m, sigma=None, sorted=None, fill_value=None):
        detections = np.argwhere(m > self.signal_to_noise * sigma).flatten()

        if len(detections) == len(m):
            raise ValueError("There is no any non-detections")

        first_detection_idx = detections[0]

        if self.strictly_fainter:
            detection_m = m[first_detection_idx]
            upper_limits = sigma[:first_detection_idx] * self.signal_to_noise
            non_detection_less = np.count_nonzero(upper_limits < detection_m)
            return non_detection_less

        return first_detection_idx

    @property
    def names(self) -> Tuple[str]:
        return ("flux_n_non_detections_before_fd",)

    @property
    def descriptions(self) -> Tuple[str]:
        return ("number of non detections before the first detection for fluxes",)

    @property
    def size(self) -> int:
        return 1


__all__ = ("FluxNNotDetBeforeFd",)



import numpy as np


def least_squares(t, m, sigma):
    A = np.vstack([t, np.ones(len(t))]).T

    if sigma is not None:
        w = np.diag(1 / sigma)
        A = np.dot(w, A)
        m = np.dot(w, m.T)

    (slope, _), residuals, *_ = np.linalg.lstsq(A, m, rcond=None)
    return slope, residuals[0]



from dataclasses import dataclass
from typing import Collection, Union

import numpy as np

from light_curve.light_curve_ext import Extractor as _RustExtractor
from light_curve.light_curve_ext import _FeatureEvaluator as _RustBaseFeature

from ._base import BaseFeature


@dataclass()
class _PyExtractor(BaseFeature):
    features: Collection[Union[BaseFeature, _RustBaseFeature]] = ()

    def _eval(self, t, m, sigma=None):
        raise NotImplementedError("_eval is missed for _PyExtractor")

    def _eval_and_fill(self, t, m, sigma, fill_value):
        return np.concatenate([np.atleast_1d(feature(t, m, sigma, fill_value)) for feature in self.features])

    @property
    def size(self):
        return sum(feature.size for feature in self.features)


class Extractor:
    def __new__(cls, *args: Collection[Union[BaseFeature, _RustBaseFeature]]):
        if len(args) > 0 and all(isinstance(feature, _RustBaseFeature) for feature in args):
            return _RustExtractor(*args)
        else:
            return _PyExtractor(args)


__all__ = ("Extractor",)



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import Cusum


def test_cusum():
    m = [1, 2, 3, 4, 5, 5]
    feature = Cusum()
    actual = feature(np.linspace(0, 1, len(m)), m, None)
    desired = 0.408248290463863
    assert_allclose(actual, desired)



from dataclasses import dataclass

from scipy.stats.mstats import mquantiles

from ._base import BaseFeature


@dataclass()
class MagnitudePercentageRatio(BaseFeature):
    n: float = 0.4
    d: float = 0.05

    def _eval(self, t, m, sigma=None):
        n1, n2 = mquantiles(m, [self.n, 1 - self.n], alphap=0.5, betap=0.5)
        d1, d2 = mquantiles(m, [self.d, 1 - self.d], alphap=0.5, betap=0.5)
        return (n2 - n1) / (d2 - d1)

    @property
    def size(self):
        return 1


__all__ = ("MagnitudePercentageRatio",)



from dataclasses import dataclass

import numpy as np
from scipy import ndimage

from ._base_meta import BaseMetaFeature


@dataclass()
class Bins(BaseMetaFeature):
    window: float = 1.0
    offset: float = 0.0

    def transform(self, t, m, sigma=None, sorted=None, fill_value=None):
        assert self.window > 0, "Window should be a positive number."
        n = np.ceil((t[-1] - t[0]) / self.window) + 1
        j = np.arange(0, n)
        bins = j * self.window

        delta = self.window * np.floor((t[0] - self.offset) / self.window)
        time = t - self.offset - delta

        idx = np.digitize(time, bins)
        uniq_idx, nums = np.unique(idx, return_counts=True)

        new_time = uniq_idx * self.window + self.offset - self.window / 2 + delta

        weights = np.power(sigma, -2)
        s = ndimage.sum(weights, labels=idx, index=uniq_idx)
        new_magn = ndimage.sum(m * weights, labels=idx, index=uniq_idx) / s
        new_sigma = np.sqrt(nums / s)

        return new_time, new_magn, new_sigma


__all__ = ("Bins",)



from itertools import product

try:
    from contextlib import nullcontext
except ImportError:
    from contextlib import contextmanager

    @contextmanager
    def nullcontext(enter_result=None):
        yield enter_result


import numpy as np
import pytest
from numpy.testing import assert_allclose, assert_array_equal

from light_curve.light_curve_ext import DmDt


def random_lc(n, sigma=True, rng=None, dtype=np.float64):
    rng = np.random.default_rng(rng)
    t = np.sort(np.asarray(rng.uniform(0, 10, n), dtype=dtype))
    m = np.asarray(rng.normal(0, 1, n), dtype=dtype)
    lc = (t, m)
    if sigma:
        sigma = np.asarray(rng.uniform(0.01, 0.1, n), dtype=dtype)
        lc += (sigma,)
    return lc


def sine_lc(n, sigma=True, dtype=np.float64):
    t = np.asarray(np.linspace(0, 10, n), dtype=dtype)
    m = np.sin(t)
    lc = (t, m)
    if sigma:
        sigma = np.full_like(t, 0.1)
        lc += (sigma,)
    return lc


def sliced(a, step=2):
    """Mix with random data and slice to original data"""
    assert step > 0
    n = a.size
    rng = np.random.default_rng()
    random_data = np.asarray(rng.normal(0, 1, (step - 1, n)), dtype=a.dtype)
    mixed = np.vstack([a[::-1], random_data]).T.reshape(-1).copy()
    s = mixed[-step::-step]
    assert not s.flags.owndata
    assert_array_equal(s, a)
    return s


def test_dmdt_count_dt_three_obs():
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=np.log10(3), max_abs_dm=1, lgdt_size=2, dm_size=32, norm=[])

    t = np.array([0, 1, 2], dtype=np.float32)

    desired = np.array([2, 1])
    actual = dmdt.count_dt(t)

    assert_array_equal(actual, desired)


def test_log_linear_grids():
    lc = random_lc(101)

    min_lgdt = -1
    max_lgdt = 1
    lgdt_size = 32
    max_abs_dm = 2
    dm_size = 32

    min_dt = 10**min_lgdt
    max_dt = 10**max_lgdt

    dt_grid = np.logspace(min_lgdt, max_lgdt, lgdt_size + 1)
    dm_grid = np.linspace(-max_abs_dm, max_abs_dm, dm_size + 1)

    dmdt_from_borders = DmDt.from_borders(
        min_lgdt=min_lgdt, max_lgdt=max_lgdt, max_abs_dm=max_abs_dm, lgdt_size=lgdt_size, dm_size=dm_size
    )
    dmdt_auto = DmDt(dt=dt_grid, dm=dm_grid, dt_type="auto", dm_type="auto")
    dmdt_log_linear = DmDt(dt=dt_grid, dm=dm_grid, dt_type="log", dm_type="linear")
    dmdt_asis = DmDt(dt=dt_grid, dm=dm_grid, dt_type="asis", dm_type="asis")

    for dmdt in (
        dmdt_from_borders,
        dmdt_auto,
        dmdt_log_linear,
        dmdt_asis,
    ):
        assert_allclose(dmdt.min_dt, min_dt)
        assert_allclose(dmdt.max_dt, max_dt)
        assert_allclose(dmdt.min_dm, -max_abs_dm)
        assert_allclose(dmdt.max_dm, max_abs_dm)
        assert_allclose(dmdt.dt_grid, dt_grid)
        assert_allclose(dmdt.dm_grid, dm_grid)

    points = dmdt_from_borders.points(lc[0], lc[1])
    gausses = dmdt_from_borders.gausses(*lc)
    for dmdt in (
        dmdt_auto,
        dmdt_log_linear,
        dmdt_asis,
    ):
        assert_allclose(dmdt.points(lc[0], lc[1]), points)
        assert_allclose(dmdt.gausses(*lc), gausses)


@pytest.mark.parametrize("lc", [sine_lc(101), sine_lc(101, dtype=np.float32)])
def test_dmdt_count_dt_contiguous_non(lc):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=[])
    desired = dmdt.count_dt(lc[0])
    actual = dmdt.count_dt(sliced(lc[0]))
    assert_array_equal(actual, desired)


@pytest.mark.parametrize("lc", [sine_lc(101), random_lc(101), sine_lc(101, dtype=np.float32)])
def test_dmdt_count_dt_many_one(lc):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=[])

    desired = dmdt.count_dt(lc[0])
    assert np.any(desired != 0)
    actual = dmdt.count_dt_many([lc[0]])

    assert actual.shape[0] == 1
    assert_array_equal(actual[0], desired)


@pytest.mark.parametrize(
    "lcs",
    [
        [sine_lc(101), sine_lc(11)],
        [random_lc(101), random_lc(101), random_lc(11)],
        [sine_lc(101, dtype=np.float32), sine_lc(11, dtype=np.float32)],
    ],
)
def test_dmdt_count_dt_many(lcs):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=[])

    desired = [dmdt.count_dt(t) for t, *_ in lcs]
    actual = dmdt.count_dt_many([t for t, *_ in lcs])

    assert_array_equal(actual, desired)


def test_dmdt_points_three_obs():
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=np.log10(3), max_abs_dm=3, lgdt_size=2, dm_size=4, norm=[])

    t = np.array([0, 1, 2], dtype=np.float32)
    m = np.array([0, 1, 2], dtype=np.float32)

    desired = np.array(
        [
            [0, 0, 2, 0],
            [0, 0, 0, 1],
        ]
    )
    actual = dmdt.points(t, m)

    assert_array_equal(actual, desired)


@pytest.mark.parametrize("lc", [sine_lc(101, False), sine_lc(101, False, dtype=np.float32)])
def test_dmdt_points_contiguous_non(lc):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=[])
    desired = dmdt.points(*lc)
    t, m = lc
    t = sliced(t)
    m = sliced(m)
    actual = dmdt.points(t, m)
    assert_array_equal(actual, desired)


@pytest.mark.parametrize("lc", [sine_lc(101, False), random_lc(101, False), sine_lc(101, False, dtype=np.float32)])
@pytest.mark.parametrize("norm", [[], ["dt"], ["max"], ["dt", "max"]])
def test_dmdt_points_many_one(lc, norm):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=norm)

    desired = dmdt.points(*lc)
    assert np.any(desired != 0)
    actual = dmdt.points_many([lc])

    assert actual.shape[0] == 1
    assert_array_equal(actual[0], desired)


@pytest.mark.parametrize(
    "lcs",
    [
        [sine_lc(101, False), sine_lc(11, False)],
        [random_lc(101, False), random_lc(101, False), random_lc(11, False)],
        [sine_lc(101, False, dtype=np.float32), sine_lc(11, False, dtype=np.float32)],
    ],
)
@pytest.mark.parametrize("norm", [[], ["dt"], ["max"], ["dt", "max"]])
def test_dmdt_points_many(lcs, norm):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=norm)

    desired = [dmdt.points(*lc) for lc in lcs]
    actual = dmdt.points_many(lcs)

    assert_array_equal(actual, desired)


@pytest.mark.parametrize("lc", [sine_lc(101), sine_lc(101, dtype=np.float32)])
def test_dmdt_gausses_contiguous_non(lc):
    dmdt = DmDt.from_borders(min_lgdt=-1, max_lgdt=1, max_abs_dm=2, lgdt_size=32, dm_size=32, norm=[])
    desired = dmdt.gausses(*lc)
    t, m, sigma = lc
    t = sliced(t)
    m = sliced(m)
    sigma = sliced(sigma)
    actual = dmdt.gausses(t, m, sigma)
    assert_array_equal(actual, desired)


@pytest.mark.parametrize("lc", [sine_lc(101), random_lc(101), sine_lc(101, dtype=np.float32)])
@pytest.mark.parametrize("norm", [[], ["dt"], ["max"], ["dt", "max"]])
@pytest.mark.parametrize("approx_erf", [True, False])
def test_dmdt_gausses_many_one(lc, norm, approx_erf):
    dmdt = DmDt.from_borders(
        min_lgdt=-1,
        max_lgdt=1,
        max_abs_dm=2,
        lgdt_size=32,
        dm_size=32,
        norm=norm,
        approx_erf=approx_erf,
    )

    desired = dmdt.gausses(*lc)
    assert np.any(desired != 0)
    actual = dmdt.gausses_many([lc])

    assert actual.shape[0] == 1
    assert_array_equal(actual[0], desired)


@pytest.mark.parametrize(
    "lcs",
    [
        [sine_lc(101), sine_lc(11)],
        [random_lc(101), random_lc(101), random_lc(11)],
        [sine_lc(101, dtype=np.float32), sine_lc(11, dtype=np.float32)],
    ],
)
@pytest.mark.parametrize("norm", [[], ["dt"], ["max"], ["dt", "max"]])
@pytest.mark.parametrize("approx_erf", [True, False])
def test_dmdt_gausses_many(lcs, norm, approx_erf):
    dmdt = DmDt.from_borders(
        min_lgdt=-1,
        max_lgdt=1,
        max_abs_dm=2,
        lgdt_size=32,
        dm_size=32,
        norm=norm,
        approx_erf=approx_erf,
    )

    desired = [dmdt.gausses(*lc) for lc in lcs]
    actual = dmdt.gausses_many(lcs)

    assert_array_equal(actual, desired)


@pytest.mark.parametrize("t_dtype,m_dtype", product(*[[np.float32, np.float64]] * 2))
def test_dmdt_points_dtype(t_dtype, m_dtype):
    t = np.linspace(0, 1, 11, dtype=t_dtype)
    m = np.asarray(t, dtype=m_dtype)
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=1, max_abs_dm=1, lgdt_size=2, dm_size=2, norm=[])
    if t_dtype is m_dtype:
        context = nullcontext()
    else:
        context = pytest.raises(TypeError)
    with context:
        dmdt.points(t, m)


@pytest.mark.parametrize("t_dtype,m_dtype,sigma_dtype", product(*[[np.float32, np.float64]] * 3))
def test_dmdt_gausses_dtype(t_dtype, m_dtype, sigma_dtype):
    t = np.linspace(1, 2, 11, dtype=t_dtype)
    m = np.asarray(t, dtype=m_dtype)
    sigma = np.asarray(t, dtype=sigma_dtype)
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=1, max_abs_dm=1, lgdt_size=2, dm_size=2, norm=[])
    if t_dtype is m_dtype is sigma_dtype:
        context = nullcontext()
    else:
        context = pytest.raises(TypeError)
    with context:
        dmdt.gausses(t, m, sigma)


@pytest.mark.parametrize("t1_dtype,m1_dtype,t2_dtype,m2_dtype", product(*[[np.float32, np.float64]] * 4))
def test_dmdt_points_many_dtype(t1_dtype, m1_dtype, t2_dtype, m2_dtype):
    t1 = np.linspace(1, 2, 11, dtype=t1_dtype)
    m1 = np.asarray(t1, dtype=m1_dtype)
    t2 = np.asarray(t1, dtype=t2_dtype)
    m2 = np.asarray(t1, dtype=m2_dtype)
    lcs = [(t1, m1), (t2, m2)]
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=1, max_abs_dm=1, lgdt_size=2, dm_size=2, norm=[])
    if t1_dtype is m1_dtype is t2_dtype is m2_dtype:
        context = nullcontext()
    else:
        context = pytest.raises(TypeError)
    with context:
        dmdt.points_many(lcs)
        dmdt.points_batches(lcs)


@pytest.mark.parametrize(
    "t1_dtype,m1_dtype,sigma1_dtype,t2_dtype,m2_dtype,sigma2_dtype", product(*[[np.float32, np.float64]] * 6)
)
def test_dmdt_gausses_many_dtype(t1_dtype, m1_dtype, sigma1_dtype, t2_dtype, m2_dtype, sigma2_dtype):
    t1 = np.linspace(1, 2, 11, dtype=t1_dtype)
    m1 = np.asarray(t1, dtype=m1_dtype)
    sigma1 = np.asarray(t1, dtype=sigma1_dtype)
    t2 = np.asarray(t1, dtype=t2_dtype)
    m2 = np.asarray(t1, dtype=m2_dtype)
    sigma2 = np.asarray(t1, dtype=sigma2_dtype)
    lcs = [(t1, m1, sigma1), (t2, m2, sigma2)]
    dmdt = DmDt.from_borders(min_lgdt=0, max_lgdt=1, max_abs_dm=1, lgdt_size=2, dm_size=2, norm=[])
    if t1_dtype is m1_dtype is sigma1_dtype is t2_dtype is m2_dtype is sigma2_dtype:
        context = nullcontext()
    else:
        context = pytest.raises(TypeError)
    with context:
        dmdt.gausses_many(lcs)
        dmdt.gausses_batches(lcs)



import numpy as np

from ._base import BaseFeature
from ._lstsq import least_squares


class LinearFit(BaseFeature):
    def _eval(self, t, m, sigma=None):
        n = len(t)

        slope, chi2 = least_squares(t, m, sigma)

        red_chi2 = chi2 / (n - 2)

        weighted_t2 = np.average(t**2, weights=np.power(sigma, -2))
        weighted_t = np.average(t, weights=np.power(sigma, -2)) ** 2

        sigma_sum = np.sum(1 / sigma**2)

        return np.array([slope, np.sqrt(1 / ((weighted_t2 - weighted_t) * sigma_sum)), red_chi2])

    @property
    def size(self):
        return 3


__all__ = ("LinearFit",)



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import MeanVariance


def test_meanvar():
    feature = MeanVariance()
    m = [1, 1, 2, 2, 3, 3, 4, 4]
    actual = feature(np.linspace(0, 1, len(m)), m, None)
    desired = 0.47809144373375745
    assert_allclose(actual, desired)



from abc import ABC, abstractmethod
from dataclasses import dataclass

import numpy as np

from light_curve.light_curve_py.warnings import mark_experimental, warn_experimental


@dataclass
class BaseFeature(ABC):
    @staticmethod
    def _normalize_input(*, t, m, sigma, sorted, check):
        t = np.asarray(t)
        m = np.asarray(m)
        if sigma is not None:
            sigma = np.asarray(sigma)
        if check:
            if np.any(~np.isfinite(t)):
                raise ValueError("t values must be finite")
            if np.any(~np.isfinite(m)):
                raise ValueError("m values must be finite")
            if sigma is not None and np.any(np.isnan(sigma)):
                raise ValueError("sigma must have no NaNs")
        if sorted is None:
            diff = np.diff(t)
            if np.any(diff == 0):
                raise ValueError("t must be unique")
            if np.any(diff < 0):
                raise ValueError("t must be sorted")
        elif not sorted:
            idx = np.argsort(t)
            t = t[idx]
            m = m[idx]
            if sigma is not None:
                sigma = sigma[idx]

        return t, m, sigma

    def _eval_and_fill(self, t, m, sigma, fill_value):
        try:
            a = self._eval(t, m, sigma)
            if np.any(~np.isfinite(a)):
                raise ValueError
            return a
        except (ValueError, ZeroDivisionError) as e:
            if fill_value is not None:
                return np.full(self.size, fill_value)
            raise e

    def __call__(self, t, m, sigma=None, sorted=None, check=True, fill_value=None):
        t, m, sigma = self._normalize_input(t=t, m=m, sigma=sigma, sorted=sorted, check=check)
        return self._eval_and_fill(t, m, sigma, fill_value)

    @mark_experimental()
    def __post_init__(self):
        pass

    def many(self, lcs, sorted=None, check=True, fill_value=None, n_jobs=-1):
        """Extract features in bulk

        This exists for computability only and doesn't support parallel
        execution, that's why `n_jobs=1` must be used
        """
        if n_jobs != 1:
            raise NotImplementedError("Parallel execution is not supported by this feature, use n_jobs=1")
        return np.stack([self(*lc, sorted=sorted, check=check, fill_value=fill_value) for lc in lcs])

    @property
    @abstractmethod
    def size(self):
        pass

    @abstractmethod
    def _eval(self, t, m, sigma=None):
        pass



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import ExcessVariance


def test_excvar():
    m = [1, 1, 2, 3, 4, 5]
    sigma = [0.5, 0.5, 0.5, 0.5, 0.5, 0.2]
    feature = ExcessVariance()
    actual = feature(np.linspace(0, 1, len(m)), m, sigma)
    desired = 0.344765625
    assert_allclose(actual, desired)



import functools
import warnings


class ExperimentalWarning(UserWarning):
    pass


def warn_experimental(msg):
    warnings.warn(msg, category=ExperimentalWarning, stacklevel=2)


def mark_experimental(msg=None):
    def inner(f):
        message = msg
        if message is None:
            full_name = f"{f.__module__}.{f.__name__}"
            message = f"Function {full_name} is experimental and may cause any kind of troubles"

        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            warn_experimental(message)
            return f(*args, **kwargs)

        return wrapped

    return inner


__all__ = (
    "ExperimentalWarning",
    "warn_experimental",
    "mark_experimental",
)



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import Skew


def test_skew():
    m = [1.0, 2.0, 3.0, 50.0, 25.0]
    feature = Skew()
    actual = feature(np.linspace(0, 1, len(m)), m, None)
    desired = 1.307253786
    assert_allclose(actual, desired)



import numpy as np

from ._base import BaseFeature


class MeanVariance(BaseFeature):
    def _eval(self, t, m, sigma=None):
        return np.std(m, ddof=1) / np.mean(m)

    @property
    def size(self):
        return 1


__all__ = ("MeanVariance",)



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import WeightedMean


def test_weightmean():
    a = [2.0, 3.0, 1.0, 9.0, 5.0]
    b = [0.3, 0.4, 0.5, 1, 1]
    feature = WeightedMean()
    actual = feature(np.linspace(0, 1, len(a)), a, b)
    desired = 2.52437574316
    assert_allclose(actual, desired)



import numpy as np
from numpy.testing import assert_allclose

from light_curve.light_curve_py import MedianBufferRangePercentage


def test_medbufrperc():
    m = np.arange(1.0, 8.0)
    feature = MedianBufferRangePercentage()
    actual = feature(m, m, None)
    desired = 1 / 7
    assert_allclose(actual, desired)



import numpy as np

from ._base import BaseFeature


class MaximumSlope(BaseFeature):
    def _eval(self, t, m, sigma=None):
        m_span = np.subtract(m[1:], m[:-1])
        t_span = np.subtract(t[1:], t[:-1])
        div = np.abs(np.divide(m_span, t_span))
        return np.amax(div)

    @property
    def size(self):
        return 1


__all__ = ("MaximumSlope",)
